//Javascript function decimalInchToArchitect
/*	Converts from decimal inches (as a Float variable) to a string representing feet, inches, and fraction.
		Second parameter sets the precision (denominator) of the fraction - commonly a power of two to give halves, quarters, eighths etc up to sixtyfourths of an inch.

	(c) John McClenahan
    January 2013
    Permission is freely give to use, copy and amend this code, provide this copyright is included in full
 */

function replaceAll(txt, original, replacement) {
  return txt.replace(new RegExp(original, 'g'),replacement);
}


function gcd(x, y) {
    while (y !== 0) {
		var z = x % y;
		x = y;
		y = z;
	}
	return x;
}

function decimalInchToArchitect(input,precision) {
//	alert("decimalInchToArchitect called");
// If precision not specified, use eights of an inch
if (!precision) { precision = 8;}	

// Check for valid input - disallow anything which is not (^) a digit 0-9, minus sign or decimal point 
var inputString = input.toString().trim();
var invalidChars = /[^0-9\-\.]/; 
// //alert ("Invalid characters are: " + invalidChars);

var notvalid = inputString.match(invalidChars);

if (notvalid) {
	alert("Invalid characters entered - must be minus sign, digit, or decimal point");
	return "";
}else{
	//alert("Input is valid");
}
// Define local variables
	// Check sign of input value
		var negative = false; 
	// Set to true if input is negative, and make input positive
		if (input <0.0) {
			negative = true;
			input = -input;
		}
	// Remove anything after the decimal to get whole number of inches
		var inches = Math.floor(input);
		//alert("Whole inches = "+ inches);
// Fraction is what's left over
		var fracn = input - inches;
		//alert("fracn = " + fracn);
	// Calculate no. of feet
		var feet = Math.floor(inches/12); // Whole number of feet
		inches = inches%12; // Remainder (%) after dividing by 12 is inches left over
		var fraction = ""; // Empty initial string for fraction
		var numerator = Math.round(fracn * precision); // Fractional inches numerator
		var denominator = precision; // Starting denominator
		var denom;
		var result = "";


// Check if denominator is a power of two (up to 64) and warn if not
		denom = (precision == 2 || precision == 4 || precision == 8 || precision == 16 || precision == 32 || precision == 64);
		// alert("denom = " +  denom);
		
		if(!denom) { // Denominator isn't regular woodworking fraction
			ok = confirm("Fraction isn't the usual woodworking half, quarter, eighth, sixteenth etc. " +
						  " Did you mean this?",false);
			if (!ok) {
				//alert("aborted");
				return "";
			}else{ // Denominator isn't a power of two. 
				// Find GCD (greatest common denominator) of numerator and denominator to simplify fraction
				if (numerator >0){
					var gd = gcd(numerator,denominator);
					numerator = numerator/gd;
					denominator = denominator/gd;
					//alert("numerator = " + numerator + " denominator = " + denominator);
					fraction = numerator.toString() + "/" + denominator.toString();
					//alert("fraction = " + fraction);
				}
			}
			
		}else{ // We have a regular woodworking fraction
			//alert("Negative result? " + negative);
			//alert("We have an OK fraction");
			
			if (numerator > 0) {
			// Simplify woodworking fraction, if there is one
				while (numerator%2 === 0) { // % x is remainder after dividing by x
					numerator = numerator/2;
					denominator = denominator/2;
				}
				if(numerator != denominator){ 
					fraction = numerator.toString() + "/" + denominator.toString();
				}else{ // we've rounded up to a whole inch, or possibly foot
					inches = inches + 1;
					numerator = 0;
					if (inches == 12) {
						feet = feet + 1;
						inches = 0;
						fraction = "";
					}
				}
			}else{
				fraction = "";
			}
		}
		
		
		if (feet > 0) { // if the length is more than a foot
			result = feet + "' "; // Add footmark and space after value for feet
		}
		//alert("5. Result feet = " + result);

		if (inches > 0) {
			result = result + inches + " " + fraction;
		//	alert("6. Inches > 0 result = " + result);
		}
		
		if (inches === 0){
			result = (result + "0 " + fraction);// Replace "0" by " " if you want whole feet to display without 0 inches
		//	alert ("7. Inches = 0 result = " + result);
		}
        
		if (feet === 0 && inches === 0 && fraction.length === 0){
			//alert("4. Result is zero " + result);
			result = ""; // Set as an empty string. Change to "0" if you want to show zero
		}
		
        // Clean up string to remove extra spaces
		
		result =  replaceAll(result," +"," ").trim();
		// alert("8. result after replaceAll = " + result);		
		// Add inchmark, unless there are only feet
		// if (inches > 0 || numerator > 0) {result = result + "\"";}
		if (negative) {result = "-" + result;} // Add minus sign if value was negative
		
		//alert("9. Final result = " + result);
		return result;

}

