// JavaScript Document ft_in_fraction.js
/* 	Converts from architectural dimensions in feet, inches and fractions, to decimal inches (as Float variable)
		Accepts most variations that make sense to the human eye
		
		Examples: 
		27' 3 3/4"
		-27'
		27' 7/8
		27 1/4" (= 2' 3 1/4")
		11 27/64"
		- 3/8"
		6.5 (= 6 1/2")
		
		Will accept common 'woodworking' fractions from 1/2 to sixtyfourths of an inch
		Will warn with JS alert, but accept, any other denominators
		Will accept feet and/or inches with decimals (but not tested extensively)
		Will accept negative numbers (precede them by a minus sign) and makes the output a negative number
		Will accept inch dimensions without feet, even if larger than one foot
		
		You can use (case insensitive) ft, ft., foot, or feet, instead of apostrophe footmark, 
		and in, in., inch, or inches instead of double quote for inchmark, or omit inchmark altogether
		
		Not sensitive to spacing - extra spaces at beginning, end, or in the middle are ignored. At least one space required between inches (if present) and start of fraction (if also present).
		
		However, don't use hyphen separators between feet and inches, or between inches and fraction.
				
        (c) John McClenahan
        January 2013
        Permission is freely give to use, copy and amend this code, provide this copyright is included in full
 */

// Define function replaceAll for use later
function replaceAll(txt, original, replacement) {
  return txt.replace(new RegExp(original, 'g'),replacement);
}

function ftInFraction(input) {
//alert("ftInFraction function called");
// Declare local variables
    var negative = false;
    var feet = 0.0;
	var inches = 0.0;
	var fraction = 0.0;
	var decimal = 0.0;
    var numerator;
    var denominator;
    var result;
    var ftmark_pos;
    var slash_char_pos;

// Tidy up input to make it consistently formatted
// Replace any footmark by ' plus space. (Also allows a single space before footmark in input)
input = input.replace(/(\'| feet| foot|feet|foot| ft\.|ft\.| ft|ft)/i,"' ");

// Remove inchmark if any
input = input.replace(/(\"|inches|inch|in.|in)/i,"");

// Replace multiple spaces by single space and trim any remaining leading or trailing spaces
input = replaceAll(input," +"," ").trim();


// Check for input to contain only valid characters digits, space, foot mark, inchmark or fraction separator /
// Any characters that are not (^ = negation) digits, space, single or double quote, or fraction divider /
invalidChars = /[^\-\0-9 \'\"\/\.]/; 

result = input.match(invalidChars);
// alert("result = " + result);

if (result) {
    alert("Invalid character(s) in string: " + result + "\nPlease re-enter your length" );
}else{
	// alert ("String contains only allowed characters: reformatted input =" + input);
	
	// Remove any remaining spaces around / character
	input = input.replace(/ \//,"/");
	input = input.replace(/\/ /,"/");
	
	previous = input; // Save previous input to redisplay if trying again
	
	// Start to parse validated and formatted string

	
	// Check if input is negative. Note this, and remove minus sign and any space after it
	if (input.substring(0,1) == "-"){ 
		negative = true;
		input = input.substring(1).trim();
	//	alert ("Input is negative. Remaining input = " + input);
	}
	
	// Find position of footmark if there is one
	ftmark_pos = input.indexOf("'");
	//alert("Footmark is at position = " + ftmark_pos);
	
	// If there is a footmark, find numeric value of feet
	if (ftmark_pos > -1) { // There are feet to find
		feet = parseFloat(input.substring(0,ftmark_pos));
		input = input.substring(ftmark_pos+2); // rest of string after footmark
		//	alert("There are " + feet + " feet");
	}
	if (input) { // there's something left for inches and/or fraction
		// See if there is a fraction
		slash_char_pos = input.indexOf("/");
		if (slash_char_pos > -1){ // there's a fraction
			space_pos = input.indexOf(" "); 
			if (space_pos >-1) { // there are also whole inches before the fraction
				inches = parseFloat(input.substring(0,space_pos + 1));
				numerator = parseFloat(input.substring(space_pos,slash_char_pos));
				denominator = parseFloat(input.substring(slash_char_pos + 1));
			}else{ // Just a fraction, no whole inches
				inches = 0.0;
				numerator = input.substring(0,slash_char_pos);
				denominator = input.substring(slash_char_pos + 1);
			}
			fraction = numerator/denominator;
			//alert("There are " + inches + " inches");
			//alert("Fraction has numerator = " + numerator + " and denominator = " + denominator + ", value = " + fraction);
			
		}else{ // There's no fraction
			//alert("No fraction found");
			inches = parseFloat(input);
			//alert("There are " + inches + " inches");
		}
	}
	
	
	if (!negative){
		decimal = feet*12.0 + inches + fraction;
	}else{
		decimal = -1.0*(feet*12.0 + inches + fraction);
	}

} // end processing valid input

	return decimal;
}